/*
 *   This file is part of Clinica.
 *
 *   Clinica is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Clinica is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Clinica.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Authors: Leonardo Robol <leo@robol.it>
 *            Gianmarco Brocchi <brocchi@poisson.phc.unipi.it>
 */
 
using Gtk;

namespace Clinica {

	public class UserInterface : GLib.Object {
	
	    /* Error signal and associated callback passed by main () */
	    public signal void error (string message);
	    
	    /* true if the graphic has been started */
	    public bool started { get; private set; default = false; }
	
	    /* Various objects needed for coordination */
		private Builder builder;	
		public Window window;

        /* Gtk Objects */
		private Alignment main;
		
		/* Pages to be loaded */
		private StartPage start_page;
		private PatientListPage patient_list_page;
		private DoctorListPage  doctor_list_page;
			
		public UserInterface () {
		
			/* Dirty hack to make user_interface available
			 * to subpages before the constructur returns */
			Application.user_interface = this;
			
		    /* Keep resource_manager and connect callback signal */
		    ResourceManager resource_manager = Application.resources;
		    error.connect ((t,l) => Application.resources.error_callback(t,l));
		    	    
		    /* Start building the interface */
			builder = new Builder ();
			
			try {
			    /* Load window-related objects and the button for the main
			     * window shown at startup */
    			builder.add_from_file (resource_manager.get_ui_file ("window.glade"));			
    			
    			/* Connect callback for destroy events */
	    		window = builder.get_object ("window") as Gtk.Window;
	        } catch (Error e) {
	            error ("Error opening UI files, check your installation.");
	        }

            /* Load the main alignement where all the pages will be put */
	        main = builder.get_object ("main") as Alignment;
	        builder.connect_signals (this);
	        
	        /* Load pages and connect callbacks with errors */
	        start_page = new StartPage ();
	        patient_list_page = new PatientListPage ();
	        doctor_list_page = new DoctorListPage ();
	        
	        /* Show starting page */
	        show_start_page ();
		}
			
		public void error_callback (Page page, string message) {
			Application.resources.error_callback(page, message);
		}
		
		/**
		 * @brief Set active page in the main window.
		 * @param page the page to be set as active.
		 */
		private void set_page (Page page) {
		    if (main.get_child () != null) {
		        if (main.get_child () == (page as Widget))
		            return;
		        else {
		        	main.get_child ().hide ();	
		            main.remove (main.get_child ());
		        }
		    } 
	        main.add (page);
            page.show ();
            page.setup ();
		}
		
		/**
		 * @brief Show the button "Create new Patient"
		 * and nothing more on the main window. 
		 */
		public void show_start_page (string patient_name = "") {
		    set_page (start_page);
		    start_page.set_name (patient_name);
		}
		
		/**
		 * @brief Show patient list in the main page.
		 */
		public void show_patient_list_page () {
		    set_page (patient_list_page);
		}
		
		/**
		 * @brief Show doctor list in the main page.
		 */
		public void show_doctor_list_page (GLib.Object? source = null) {
		    set_page (doctor_list_page);
		}
		
		/* CALLBACKS */
	    [CCode (instance_pos = -1)]
		public void on_window_destroy (GLib.Object w) {
			Gtk.main_quit ();
		}
		
		[CCode (instance_pos = -1)]
		public void on_browse_patient_list_requested (StartPage page) {
		    show_patient_list_page ();
		}
		
		[CCode (instance_pos = -1)]
		public void on_browse_doctor_list_requested (StartPage page) {
		    show_doctor_list_page ();
		}
		
		/* Menu item */
		
		[CCode (instance_pos = -1)]
		public void on_new_patient_menu_item_activate (MenuItem item) {
			var new_patient_dialog = new PatientEditor ();
			new_patient_dialog.run ();
			new_patient_dialog.destroy ();
		}
		
		[CCode (instance_pos = -1)]
        public void on_new_doctor_menu_item_activate (MenuItem item) {
        	var new_doc_dialog = new DoctorEditor ();
        	new_doc_dialog.run ();
			new_doc_dialog.destroy ();
        }
		
		[CCode (instance_pos = -1)]
		public void on_patient_menu_item_activate (MenuItem item) {
		    show_patient_list_page ();
		}
		
		[CCode (instance_pos = -1)]
		public void on_doctor_menu_item_activate (MenuItem item) {
		    show_doctor_list_page ();
		}
		
		[CCode (instance_pos = -1)]
		public void on_home_menu_item_activated (MenuItem item) {
		    show_start_page();
		} 
		
		[CCode (instance_pos = -1)]
		public void on_about_menu_item_activated (GLib.Object? source) {
		    var about_dialog = new AboutDialog ();
		    about_dialog.run ();
		    about_dialog.destroy ();
		}
		
		/**
		 * @brief start application
		 */
		public void start () {
			window.show_all ();
			started = true;
			Gtk.main ();
		}
	}
}
